from typing import Dict, List, Literal, Optional

from datetime import datetime

from pydantic import Extra, validator

from paddle_billing_client.models import LazyBaseModel as BaseModel
from paddle_billing_client.models import PaddleResponse
from paddle_billing_client.models.price import Price


class ProductBase(BaseModel):
    name: str
    tax_category: Literal[
        "digital-goods",
        "ebooks",
        "implementation-services",
        "professional-services",
        "saas",
        "software-programming-services",
        "standard",
        "training-services",
        "website-hosting",
    ]
    description: Optional[str]
    image_url: Optional[str]
    custom_data: Optional[Dict[str, str]]
    status: Optional[Literal["active", "archived"]]


class Product(ProductBase):
    id: str
    created_at: Optional[datetime]
    prices: Optional[List[Price]]


class ProductQueryParams(BaseModel):
    # Return entities after the specified cursor. Used for working through paginated results.
    after: Optional[str] = None
    # Return only the IDs specified. Use a comma separated list to get multiple entities.
    id: Optional[str] = None
    # Include related entities in the response.
    include: Optional[Literal["prices"]] = None
    # Order returned entities by the specified field and direction ([ASC] or [DESC]).
    order_by: Optional[Literal["[ASC]", "[DESC]"]] = None
    # Set how many entities are returned per page. Default: 50
    per_page: Optional[int] = None
    # Return entities that match the specified status. Use a comma separated list to specify multiple status values.
    status: Optional[str] = None
    # Return entities that match the specified tax category.
    tax_category: Optional[
        Literal[
            "digital-goods",
            "ebooks",
            "implementation-services",
            "professional-services",
            "saas",
            "software-programming-services",
            "standard",
            "training-services",
            "website-hosting",
        ]
    ] = None

    class Config:
        extra = Extra.forbid

    @validator("status", allow_reuse=True)
    def check_status(cls, v: str) -> str:  # pragma: no cover
        valid_statuses = ["active", "archived"]
        if not all([s in valid_statuses for s in v.split(",")]):
            raise ValueError(
                f"Query param invalid status: {v}, allowed values: {valid_statuses}"
            )
        return v


class ProductResponse(PaddleResponse):
    data: Product


class ProductsResponse(PaddleResponse):
    data: List[Product]


class ProductRequest(ProductBase):
    pass



from __future__ import annotations

from apiclient import APIClient, JsonResponseHandler
from apiclient_pydantic import serialize_all_methods

from paddle_billing_client.endpoints import Endpoints
from paddle_billing_client.formatters import CustomJsonRequestFormatter
from paddle_billing_client.models.address import (
    AddressesResponse,
    AddressQueryParams,
    AddressRequest,
    AddressResponse,
)
from paddle_billing_client.models.adjustment import (
    AdjustmentQueryParams,
    AdjustmentRequest,
    AdjustmentResponse,
    AdjustmentsResponse,
)
from paddle_billing_client.models.business import (
    BusinessesResponse,
    BusinessQueryParams,
    BusinessRequest,
    BusinessResponse,
)
from paddle_billing_client.models.common import Paginate
from paddle_billing_client.models.customer import (
    CustomerBalancesQueryParams,
    CustomerBalancesResponse,
    CustomerQueryParams,
    CustomerRequest,
    CustomerResponse,
    CustomersResponse,
)
from paddle_billing_client.models.discount import (
    DiscountQueryParams,
    DiscountRequest,
    DiscountResponse,
    DiscountsResponse,
)
from paddle_billing_client.models.event import EventsResponse, EventTypesResponse
from paddle_billing_client.models.notification import (
    NotificationQueryParams,
    NotificationReplayResponse,
    NotificationResponse,
    NotificationsResponse,
)
from paddle_billing_client.models.notification_setting import (
    NotificationSettingRequest,
    NotificationSettingResponse,
    NotificationSettingsResponse,
)
from paddle_billing_client.models.price import (
    PriceQueryParams,
    PriceRequest,
    PriceResponse,
    PricesResponse,
)
from paddle_billing_client.models.product import (
    ProductQueryParams,
    ProductRequest,
    ProductResponse,
    ProductsResponse,
)
from paddle_billing_client.models.subscription import (
    SubscriptionQueryParams,
    SubscriptionRequest,
    SubscriptionResponse,
    SubscriptionsResponse,
)
from paddle_billing_client.models.transaction import (
    TransactionPdfResponse,
    TransactionPreviewResponse,
    TransactionQueryParams,
    TransactionRequest,
    TransactionResponse,
    TransactionsResponse,
)


@serialize_all_methods()
class PaddleApiClient(APIClient):
    def __init__(
        self,
        base_url="https://sandbox-api.paddle.com",
        request_formatter=CustomJsonRequestFormatter,
        response_handler=JsonResponseHandler,
        **kwargs,
    ):
        self.endpoints: Endpoints = Endpoints(base_url=base_url)
        super().__init__(
            request_formatter=request_formatter,
            response_handler=response_handler,
            **kwargs,
        )

    """
    Products
    """

    def create_product(self, data: ProductRequest) -> ProductResponse:
        """Create a product"""
        return self.post(self.endpoints.create_product, dict(data))

    def get_product(self, product_id: str) -> ProductResponse:
        """Get a product"""
        return self.get(self.endpoints.get_product.format(product_id=product_id))

    def list_products(
        self, query_params: ProductQueryParams = None, paginate: Paginate = None
    ) -> ProductsResponse:
        """List all products"""
        return self.get(
            dict(paginate)["next"] if paginate else self.endpoints.list_products,
            params=dict(query_params),
        )

    def update_product(self, product_id: str, data: ProductRequest) -> ProductResponse:
        """Update a product"""
        return self.patch(
            self.endpoints.update_product.format(product_id=product_id), dict(data)
        )

    """
    Prices
    """

    def create_price(self, data: PriceRequest) -> PriceResponse:
        """Create a price"""
        return self.post(self.endpoints.create_price, dict(data))

    def get_price(self, price_id: str) -> PriceResponse:
        """Get a price"""
        return self.get(self.endpoints.get_price.format(price_id=price_id))

    def list_prices(
        self, query_params: PriceQueryParams = None, paginate: Paginate = None
    ) -> PricesResponse:
        """List all prices"""
        return self.get(
            dict(paginate)["next"] if paginate else self.endpoints.list_prices,
            params=dict(query_params),
        )

    def update_price(self, price_id: str, data: PriceRequest) -> PriceResponse:
        """Update a price"""
        return self.patch(
            self.endpoints.update_price.format(price_id=price_id), dict(data)
        )

    """
    Discounts
    """

    def create_discount(self, data: DiscountRequest) -> DiscountResponse:
        """Create a discount"""
        return self.post(self.endpoints.create_discount, dict(data))

    def get_discount(self, discount_id: str) -> DiscountResponse:
        """Get a discount"""
        return self.get(self.endpoints.get_discount.format(discount_id=discount_id))

    def list_discounts(
        self, query_params: DiscountQueryParams = None, paginate: Paginate = None
    ) -> DiscountsResponse:
        """List all discounts"""
        return self.get(
            dict(paginate)["next"] if paginate else self.endpoints.list_discounts,
            params=dict(query_params),
        )

    def update_discount(
        self, discount_id: str, data: DiscountRequest
    ) -> DiscountResponse:
        """Update a discount"""
        return self.patch(
            self.endpoints.update_discount.format(discount_id=discount_id), dict(data)
        )

    """
    Customers
    """

    def create_customer(self, data: CustomerRequest) -> CustomerResponse:
        """Create a customer"""
        return self.post(self.endpoints.create_customer, dict(data))

    def get_customer(self, customer_id: str) -> CustomerResponse:
        """Get a customer"""
        return self.get(self.endpoints.get_customer.format(customer_id=customer_id))

    def list_customers(
        self, query_params: CustomerQueryParams = None, paginate: Paginate = None
    ) -> CustomersResponse:
        """List all customers"""
        return self.get(
            dict(paginate)["next"] if paginate else self.endpoints.list_customers,
            params=dict(query_params),
        )

    def update_customer(
        self, customer_id: str, data: CustomerRequest
    ) -> CustomerResponse:
        """Update a customer"""
        return self.patch(
            self.endpoints.update_customer.format(customer_id=customer_id), dict(data)
        )

    def list_customer_credit_balances(
        self, query_params: CustomerBalancesQueryParams = None
    ) -> CustomerBalancesResponse:
        """List credit balances for a customer"""
        return self.get(
            self.endpoints.list_customer_credit_balances, params=dict(query_params)
        )

    """
    Addresses
    """

    def create_address_for_customer(
        self, customer_id: str, data: AddressRequest
    ) -> AddressResponse:
        """Create an address for a customer"""
        return self.post(
            self.endpoints.create_address_for_customer.format(customer_id=customer_id),
            dict(data),
        )

    def get_address_for_customer(
        self, customer_id: str, address_id: str
    ) -> AddressResponse:
        """Get an address for a customer"""
        return self.get(
            self.endpoints.get_address_for_customer.format(
                customer_id=customer_id, address_id=address_id
            )
        )

    def list_addresses_for_customer(
        self,
        customer_id: str,
        query_params: AddressQueryParams = None,
        paginate: Paginate = None,
    ) -> AddressesResponse:
        """List all addresses for a customer"""
        return self.get(
            dict(paginate)["next"]
            if paginate
            else self.endpoints.list_addresses_for_customer.format(
                customer_id=customer_id
            ),
            params=dict(query_params),
        )

    def update_address_for_customer(
        self, customer_id: str, address_id: str, data: AddressRequest
    ) -> AddressResponse:
        """Update an address for a customer"""
        return self.patch(
            self.endpoints.update_address_for_customer.format(
                customer_id=customer_id, address_id=address_id
            ),
            dict(data),
        )

    """
    Businesses
    """

    def create_business_for_customer(
        self, customer_id: str, data: BusinessRequest
    ) -> BusinessResponse:
        """Create a business for a customer"""
        return self.post(
            self.endpoints.create_business_for_customer.format(customer_id=customer_id),
            dict(data),
        )

    def get_business_for_customer(
        self, customer_id: str, business_id: str
    ) -> BusinessResponse:
        """Get a business for a customer"""
        return self.get(
            self.endpoints.get_business_for_customer.format(
                customer_id=customer_id, business_id=business_id
            )
        )

    def list_businesses_for_customer(
        self,
        customer_id: str,
        query_params: BusinessQueryParams = None,
        paginate: Paginate = None,
    ) -> BusinessesResponse:
        """List all businesses for a customer"""
        return self.get(
            dict(paginate)["next"]
            if paginate
            else self.endpoints.list_businesses_for_customer.format(
                customer_id=customer_id
            ),
            params=dict(query_params),
        )

    def update_business_for_customer(
        self, customer_id: str, business_id: str, data: BusinessRequest
    ) -> BusinessResponse:
        """Update a business for a customer"""
        return self.patch(
            self.endpoints.update_business_for_customer.format(
                customer_id=customer_id, business_id=business_id
            ),
            dict(data),
        )

    """
    Transactions
    """

    def create_transaction(
        self,
        data: TransactionRequest,
        query_params: TransactionQueryParams = None,
    ) -> TransactionResponse:
        """Create a transaction"""
        return self.post(
            self.endpoints.create_transaction, dict(data), params=dict(query_params)
        )

    def get_transaction(
        self, transaction_id: str, query_params: TransactionQueryParams = None
    ) -> TransactionResponse:
        """Get a transaction"""
        return self.get(
            self.endpoints.get_transaction.format(transaction_id=transaction_id),
            params=dict(query_params),
        )

    def list_transactions(
        self, query_params: TransactionQueryParams = None, paginate: Paginate = None
    ) -> TransactionsResponse:
        """List all transactions"""
        return self.get(
            dict(paginate)["next"] if paginate else self.endpoints.list_transactions,
            params=dict(query_params),
        )

    def update_transaction(
        self,
        transaction_id: str,
        data: TransactionRequest,
        query_params: TransactionQueryParams = None,
    ) -> TransactionResponse:
        """Update a transaction"""
        return self.patch(
            self.endpoints.update_transaction.format(transaction_id=transaction_id),
            dict(data),
            params=dict(query_params),
        )

    def preview_transaction(
        self, data: TransactionRequest, query_params: TransactionQueryParams = None
    ) -> TransactionPreviewResponse:
        """Preview a transaction"""
        return self.post(
            self.endpoints.preview_transaction, dict(data), params=dict(query_params)
        )

    def preview_prices(self, data: TransactionRequest) -> TransactionResponse:
        """Preview prices"""
        return self.post(self.endpoints.preview_prices, dict(data))

    def get_pdf_for_transaction(self, transaction_id: str) -> TransactionPdfResponse:
        """Get a PDF for a transaction"""
        return self.get(
            self.endpoints.get_pdf_for_transaction.format(transaction_id=transaction_id)
        )

    """
    Subscriptions
    """

    def get_subscription(self, subscription_id: str) -> SubscriptionResponse:
        """Get a subscription"""
        return self.get(
            self.endpoints.get_subscription.format(subscription_id=subscription_id)
        )

    def list_subscriptions(
        self, query_params: SubscriptionQueryParams = None, paginate: Paginate = None
    ) -> SubscriptionsResponse:
        """List all subscriptions"""
        return self.get(
            dict(paginate)["next"] if paginate else self.endpoints.list_subscriptions,
            params=dict(query_params),
        )

    def preview_update_subscription(
        self, subscription_id: str, data: SubscriptionRequest
    ) -> SubscriptionResponse:
        """Preview an update to a subscription"""
        return self.patch(
            self.endpoints.preview_update_subscription.format(
                subscription_id=subscription_id
            ),
            dict(data),
        )

    def update_subscription(
        self, subscription_id: str, data: SubscriptionRequest
    ) -> SubscriptionResponse:
        """Update a subscription"""
        return self.patch(
            self.endpoints.update_subscription.format(subscription_id=subscription_id),
            dict(data),
        )

    def get_transaction_to_update_payment_method(
        self, subscription_id: str
    ) -> TransactionResponse:
        """Get a transaction to update a payment method"""
        return self.get(
            self.endpoints.get_transaction_to_update_payment_method.format(
                subscription_id=subscription_id
            )
        )

    def preview_one_time_charge(
        self, subscription_id: str, data: SubscriptionRequest
    ) -> SubscriptionResponse:
        """Preview a one-time charge"""
        return self.post(
            self.endpoints.preview_one_time_charge.format(
                subscription_id=subscription_id
            ),
            dict(data),
        )

    def create_one_time_charge(
        self, subscription_id: str, data: SubscriptionRequest
    ) -> SubscriptionResponse:
        """Create a one-time charge"""
        return self.post(
            self.endpoints.create_one_time_charge.format(
                subscription_id=subscription_id
            ),
            dict(data),
        )

    def activate_trialing_subscription(
        self, subscription_id: str, data: SubscriptionRequest
    ) -> SubscriptionResponse:
        """Activate a trialing subscription"""
        return self.post(
            self.endpoints.activate_trialing_subscription.format(
                subscription_id=subscription_id
            ),
            dict(data),
        )

    def pause_subscription(
        self, subscription_id: str, data: SubscriptionRequest
    ) -> SubscriptionResponse:
        """Pause a subscription"""
        return self.post(
            self.endpoints.pause_subscription.format(subscription_id=subscription_id),
            dict(data),
        )

    def resume_subscription(
        self, subscription_id: str, data: SubscriptionRequest
    ) -> SubscriptionResponse:
        """Resume a subscription"""
        return self.post(
            self.endpoints.resume_subscription.format(subscription_id=subscription_id),
            dict(data),
        )

    def cancel_subscription(
        self, subscription_id: str, data: SubscriptionRequest
    ) -> SubscriptionResponse:
        """Cancel a subscription"""
        return self.post(
            self.endpoints.cancel_subscription.format(subscription_id=subscription_id),
            dict(data),
        )

    """
    Adjustments
    """

    def create_adjustment(self, data: AdjustmentRequest) -> AdjustmentResponse:
        """Create a customer"""
        return self.post(self.endpoints.create_adjustment, dict(data))

    def list_adjustments(
        self, query_params: AdjustmentQueryParams = None, paginate: Paginate = None
    ) -> AdjustmentsResponse:
        """List all customers"""
        return self.get(
            dict(paginate)["next"] if paginate else self.endpoints.list_adjustments,
            params=dict(query_params),
        )

    """
    Events
    """

    def list_event_types(self) -> EventTypesResponse:
        """List all customers"""
        return self.get(self.endpoints.list_event_types)

    def list_events(self) -> EventsResponse:
        """List all customers"""
        return self.get(self.endpoints.list_events)

    """
    Notification settings
    """

    def create_notification_setting(
        self, data: NotificationSettingRequest
    ) -> NotificationSettingResponse:
        """Create notification settings"""
        return self.post(self.endpoints.create_notification_setting, dict(data))

    def get_notification_setting(
        self, notification_setting_id: str
    ) -> NotificationSettingResponse:
        """Get notification settings"""
        return self.get(
            self.endpoints.get_notification_setting.format(
                notification_setting_id=notification_setting_id
            )
        )

    def list_notification_settings(self) -> NotificationSettingsResponse:
        """List notification settings"""
        return self.get(self.endpoints.list_notification_settings)

    def update_notification_setting(
        self, notification_setting_id: str, data: NotificationSettingRequest
    ) -> NotificationSettingResponse:
        """Update notification settings"""
        return self.patch(
            self.endpoints.update_notification_setting.format(
                notification_setting_id=notification_setting_id
            ),
            dict(data),
        )

    def delete_notification_setting(
        self, notification_setting_id: str
    ) -> NotificationSettingResponse:
        """Delete notification settings"""
        return self.delete(
            self.endpoints.delete_notification_setting.format(
                notification_setting_id=notification_setting_id
            )
        )

    """
    Notifications
    """

    def get_notification(self, notification_id: str) -> NotificationResponse:
        """Get a notification"""
        return self.get(
            self.endpoints.get_notification.format(notification_id=notification_id)
        )

    def list_notifications(
        self,
        query_params: NotificationQueryParams = None,
        paginate: Paginate = None,
    ) -> NotificationsResponse:
        """List all notifications"""
        return self.get(
            dict(paginate)["next"] if paginate else self.endpoints.list_notifications,
            params=dict(query_params),
        )

    def replay_notification(self, notification_id: str) -> NotificationReplayResponse:
        """Replay a notification"""
        return self.post(
            self.endpoints.replay_notification.format(notification_id=notification_id),
            dict(),
        )



from __future__ import annotations

from typing import Dict, List, Literal, Optional

from datetime import datetime

from pydantic import Extra, validator

from paddle_billing_client.models import LazyBaseModel as BaseModel
from paddle_billing_client.models import PaddleResponse


class Contact(BaseModel):
    name: str
    email: str


class BusinessBase(BaseModel):
    name: str
    company_number: str | None
    tax_identifier: str | None
    contacts: list[Contact]
    status: Literal["active", "archived"] | None
    custom_data: dict[str, str] | None


class Business(BusinessBase):
    id: str
    created_at: datetime | None
    updated_at: datetime | None


class BusinessQueryParams(BaseModel):
    # Return entities after the specified cursor. Used for working through paginated results.
    after: str | None = None
    # Return only the IDs specified. Use a comma separated list to get multiple entities.
    id: str | None = None
    # Order returned entities by the specified field and direction ([ASC] or [DESC]).
    order_by: Literal["[ASC]", "[DESC]"] | None = None
    # Set how many entities are returned per page. Default: 50
    per_page: int | None = None
    # Return entities that match a search query. Searches id and type fields.
    search: str | None = None
    # Return entities that match the specified status. Use a comma separated list to specify multiple status values.
    status: str | None = None

    class Config:
        extra = Extra.forbid

    @validator("status", allow_reuse=True)
    def check_status(cls, v: str) -> str:
        valid_statuses = ["active", "archived"]
        if not all([s in valid_statuses for s in v.split(",")]):
            raise ValueError(
                f"Query param invalid status: {v}, allowed values: {valid_statuses}"
            )
        return v


class BusinessResponse(PaddleResponse):
    data: Business


class BusinessesResponse(PaddleResponse):
    data: list[Business]


class BusinessRequest(BusinessBase):
    pass



import datetime
import json

from apiclient import JsonRequestFormatter
from apiclient.utils.typing import OptionalJsonType, OptionalStr


class CustomJsonRequestFormatter(JsonRequestFormatter):
    """Format the outgoing data as json."""

    content_type = "application/json"

    @staticmethod
    def default(o):
        if isinstance(o, (datetime.date, datetime.datetime)):
            return o.isoformat() + "Z"

    @classmethod
    def format(cls, data: OptionalJsonType) -> OptionalStr:
        if data:
            return json.dumps(data, default=cls.default)



from typing import Dict, List, Literal, Optional

from datetime import datetime

from pydantic import Extra, validator

from paddle_billing_client.models import LazyBaseModel as BaseModel
from paddle_billing_client.models import PaddleResponse


class DiscountBase(BaseModel):
    amount: str
    description: str
    type: Literal["flat", "flat_per_seat", "percentage"]
    enabled_for_checkout: bool
    code: str
    currency_code: str
    recur: bool
    maximum_recurring_intervals: Optional[int]
    usage_limit: Optional[int]
    restrict_to: Optional[List[str]]
    expires_at: Optional[datetime]
    status: Optional[Literal["active", "archived", "expired", "used"]]
    custom_data: Optional[Dict[str, str]]


class Discount(DiscountBase):
    id: str
    times_used: int
    created_at: Optional[datetime]
    updated_at: Optional[datetime]
    external_id: Optional[str]


class DiscountQueryParams(BaseModel):
    # Return entities after the specified cursor. Used for working through paginated results.
    after: Optional[str] = None
    # Return entities that match the discount code. Use a comma separated list to specify multiple discount codes.
    code: Optional[str] = None
    # Return only the IDs specified. Use a comma separated list to get multiple entities.
    id: Optional[str] = None
    # Order returned entities by the specified field and direction ([ASC] or [DESC]).
    order_by: Optional[Literal["[ASC]", "[DESC]"]] = None
    # Set how many entities are returned per page. Default: 50
    per_page: Optional[int] = None
    # Return entities that match the specified status. Use a comma separated list to specify multiple status values.
    status: Optional[str] = None

    class Config:
        extra = Extra.forbid

    @validator("status", allow_reuse=True)
    def check_status(cls, v: str) -> str:  # pragma: no cover
        valid_statuses = ["active", "archived", "expired", "used"]
        if not all([s in valid_statuses for s in v.split(",")]):
            raise ValueError(
                f"Query param invalid status: {v}, allowed values: {valid_statuses}"
            )
        return v


class DiscountResponse(PaddleResponse):
    data: Discount


class DiscountsResponse(PaddleResponse):
    data: List[Discount]


class DiscountRequest(DiscountBase):
    pass



from __future__ import annotations

from pydantic import BaseModel, Extra


class BillingPeriod(BaseModel):
    ends_at: str
    starts_at: str


class Paginate(BaseModel):
    next: str | None

    class Config:
        extra = Extra.forbid
